@@ -16,6 +16,8 @@ import (
 	bitbucketserver "gopkg.in/go-playground/webhooks.v5/bitbucket-server"
 	"gopkg.in/go-playground/webhooks.v5/github"
 	"gopkg.in/go-playground/webhooks.v5/gitlab"
+	"k8s.io/apimachinery/pkg/runtime"
+	kubetesting "k8s.io/client-go/testing"
 
 	"github.com/argoproj/argo-cd/v2/util/cache/appstate"
 
@@ -45,8 +47,22 @@ func (f fakeSettingsSrc) GetTrackingMethod() (string, error) {
 	return "", nil
 }
 
-func NewMockHandler() *ArgoCDWebhookHandler {
-	appClientset := appclientset.NewSimpleClientset()
+type reactorDef struct {
+	verb     string
+	resource string
+	reaction kubetesting.ReactionFunc
+}
+
+func NewMockHandler(reactor *reactorDef, objects ...runtime.Object) *ArgoCDWebhookHandler {
+	appClientset := appclientset.NewSimpleClientset(objects...)
+	if reactor != nil {
+		defaultReactor := appClientset.ReactionChain[0]
+		appClientset.ReactionChain = nil
+		appClientset.AddReactor("list", "*", func(action kubetesting.Action) (handled bool, ret runtime.Object, err error) {
+			return defaultReactor.React(action)
+		})
+		appClientset.AddReactor(reactor.verb, reactor.resource, reactor.reaction)
+	}
 	cacheClient := cacheutil.NewCache(cacheutil.NewInMemoryCache(1 * time.Hour))
 	return NewHandler("", appClientset, &settings.ArgoCDSettings{}, &fakeSettingsSrc{}, cache.NewCache(
 		cacheClient,
@@ -57,7 +73,7 @@ func NewMockHandler() *ArgoCDWebhookHandler {
 
 func TestGitHubCommitEvent(t *testing.T) {
 	hook := test.NewGlobal()
-	h := NewMockHandler()
+	h := NewMockHandler(nil)
 	req := httptest.NewRequest("POST", "/api/webhook", nil)
 	req.Header.Set("X-GitHub-Event", "push")
 	eventJSON, err := os.ReadFile("testdata/github-commit-event.json")
@@ -71,9 +87,63 @@ func TestGitHubCommitEvent(t *testing.T) {
 	hook.Reset()
 }
 
+// TestGitHubCommitEvent_MultiSource_Refresh makes sure that a webhook will refresh a multi-source app when at least
+// one source matches.
+func TestGitHubCommitEvent_MultiSource_Refresh(t *testing.T) {
+	hook := test.NewGlobal()
+	var patched bool
+	reaction := func(action kubetesting.Action) (handled bool, ret runtime.Object, err error) {
+		patchAction := action.(kubetesting.PatchAction)
+		assert.Equal(t, "app-to-refresh", patchAction.GetName())
+		patched = true
+		return true, nil, nil
+	}
+	h := NewMockHandler(&reactorDef{"patch", "applications", reaction}, &v1alpha1.Application{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "app-to-refresh",
+		},
+		Spec: v1alpha1.ApplicationSpec{
+			Sources: v1alpha1.ApplicationSources{
+				{
+					RepoURL: "https://github.com/some/unrelated-repo",
+					Path:    ".",
+				},
+				{
+					RepoURL: "https://github.com/jessesuen/test-repo",
+					Path:    ".",
+				},
+			},
+		},
+	}, &v1alpha1.Application{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "app-to-ignore",
+		},
+		Spec: v1alpha1.ApplicationSpec{
+			Sources: v1alpha1.ApplicationSources{
+				{
+					RepoURL: "https://github.com/some/unrelated-repo",
+					Path:    ".",
+				},
+			},
+		},
+	})
+	req := httptest.NewRequest("POST", "/api/webhook", nil)
+	req.Header.Set("X-GitHub-Event", "push")
+	eventJSON, err := os.ReadFile("testdata/github-commit-event.json")
+	assert.NoError(t, err)
+	req.Body = io.NopCloser(bytes.NewReader(eventJSON))
+	w := httptest.NewRecorder()
+	h.Handler(w, req)
+	assert.Equal(t, w.Code, http.StatusOK)
+	expectedLogResult := "Requested app 'app-to-refresh' refresh"
+	assert.Equal(t, expectedLogResult, hook.LastEntry().Message)
+	assert.True(t, patched)
+	hook.Reset()
+}
+
 func TestGitHubTagEvent(t *testing.T) {
 	hook := test.NewGlobal()
-	h := NewMockHandler()
+	h := NewMockHandler(nil)
 	req := httptest.NewRequest("POST", "/api/webhook", nil)
 	req.Header.Set("X-GitHub-Event", "push")
 	eventJSON, err := os.ReadFile("testdata/github-tag-event.json")
@@ -89,7 +159,7 @@ func TestGitHubTagEvent(t *testing.T) {
 
 func TestGitHubPingEvent(t *testing.T) {
 	hook := test.NewGlobal()
-	h := NewMockHandler()
+	h := NewMockHandler(nil)
 	req := httptest.NewRequest("POST", "/api/webhook", nil)
 	req.Header.Set("X-GitHub-Event", "ping")
 	eventJSON, err := os.ReadFile("testdata/github-ping-event.json")
@@ -105,7 +175,7 @@ func TestGitHubPingEvent(t *testing.T) {
 
 func TestBitbucketServerRepositoryReferenceChangedEvent(t *testing.T) {
 	hook := test.NewGlobal()
-	h := NewMockHandler()
+	h := NewMockHandler(nil)
 	req := httptest.NewRequest("POST", "/api/webhook", nil)
 	req.Header.Set("X-Event-Key", "repo:refs_changed")
 	eventJSON, err := os.ReadFile("testdata/bitbucket-server-event.json")
@@ -123,7 +193,7 @@ func TestBitbucketServerRepositoryReferenceChangedEvent(t *testing.T) {
 
 func TestBitbucketServerRepositoryDiagnosticPingEvent(t *testing.T) {
 	hook := test.NewGlobal()
-	h := NewMockHandler()
+	h := NewMockHandler(nil)
 	eventJSON := "{\"test\": true}"
 	req := httptest.NewRequest("POST", "/api/webhook", bytes.NewBufferString(eventJSON))
 	req.Header.Set("X-Event-Key", "diagnostics:ping")
@@ -137,7 +207,7 @@ func TestBitbucketServerRepositoryDiagnosticPingEvent(t *testing.T) {
 
 func TestGogsPushEvent(t *testing.T) {
 	hook := test.NewGlobal()
-	h := NewMockHandler()
+	h := NewMockHandler(nil)
 	req := httptest.NewRequest("POST", "/api/webhook", nil)
 	req.Header.Set("X-Gogs-Event", "push")
 	eventJSON, err := os.ReadFile("testdata/gogs-event.json")
@@ -153,7 +223,7 @@ func TestGogsPushEvent(t *testing.T) {
 
 func TestGitLabPushEvent(t *testing.T) {
 	hook := test.NewGlobal()
-	h := NewMockHandler()
+	h := NewMockHandler(nil)
 	req := httptest.NewRequest("POST", "/api/webhook", nil)
 	req.Header.Set("X-Gitlab-Event", "Push Hook")
 	eventJSON, err := os.ReadFile("testdata/gitlab-event.json")
@@ -169,7 +239,7 @@ func TestGitLabPushEvent(t *testing.T) {
 
 func TestInvalidMethod(t *testing.T) {
 	hook := test.NewGlobal()
-	h := NewMockHandler()
+	h := NewMockHandler(nil)
 	req := httptest.NewRequest("GET", "/api/webhook", nil)
 	req.Header.Set("X-GitHub-Event", "push")
 	w := httptest.NewRecorder()
@@ -183,7 +253,7 @@ func TestInvalidMethod(t *testing.T) {
 
 func TestInvalidEvent(t *testing.T) {
 	hook := test.NewGlobal()
-	h := NewMockHandler()
+	h := NewMockHandler(nil)
 	req := httptest.NewRequest("POST", "/api/webhook", nil)
 	req.Header.Set("X-GitHub-Event", "push")
 	w := httptest.NewRecorder()
@@ -197,7 +267,7 @@ func TestInvalidEvent(t *testing.T) {
 
 func TestUnknownEvent(t *testing.T) {
 	hook := test.NewGlobal()
-	h := NewMockHandler()
+	h := NewMockHandler(nil)
 	req := httptest.NewRequest("POST", "/api/webhook", nil)
 	req.Header.Set("X-Unknown-Event", "push")
 	w := httptest.NewRecorder()
@@ -215,94 +285,114 @@ func getApp(annotation string, sourcePath string) *v1alpha1.Application {
 			},
 		},
 		Spec: v1alpha1.ApplicationSpec{
-			Source: v1alpha1.ApplicationSource{
+			Source: &v1alpha1.ApplicationSource{
 				Path: sourcePath,
 			},
 		},
 	}
 }
 
+func getMultiSourceApp(annotation string, paths ...string) *v1alpha1.Application {
+	var sources v1alpha1.ApplicationSources
+	for _, path := range paths {
+		sources = append(sources, v1alpha1.ApplicationSource{Path: path})
+	}
+	return &v1alpha1.Application{
+		ObjectMeta: metav1.ObjectMeta{
+			Annotations: map[string]string{
+				v1alpha1.AnnotationKeyManifestGeneratePaths: annotation,
+			},
+		},
+		Spec: v1alpha1.ApplicationSpec{
+			Sources: sources,
+		},
+	}
+}
+
 func Test_getAppRefreshPrefix(t *testing.T) {
 	tests := []struct {
-		name  string
-		app   *v1alpha1.Application
-		files []string
-		want  bool
+		name           string
+		app            *v1alpha1.Application
+		files          []string
+		changeExpected bool
 	}{
 		{"default no path", &v1alpha1.Application{}, []string{"README.md"}, true},
 		{"relative path - matching", getApp(".", "source/path"), []string{"source/path/my-deployment.yaml"}, true},
+		{"relative path, multi source - matching #1", getMultiSourceApp(".", "source/path", "other/path"), []string{"source/path/my-deployment.yaml"}, true},
+		{"relative path, multi source - matching #2", getMultiSourceApp(".", "other/path", "source/path"), []string{"source/path/my-deployment.yaml"}, true},
 		{"relative path - not matching", getApp(".", "source/path"), []string{"README.md"}, false},
+		{"relative path, multi source - not matching", getMultiSourceApp(".", "other/path", "unrelated/path"), []string{"README.md"}, false},
 		{"absolute path - matching", getApp("/source/path", "source/path"), []string{"source/path/my-deployment.yaml"}, true},
+		{"absolute path, multi source - matching #1", getMultiSourceApp("/source/path", "source/path", "other/path"), []string{"source/path/my-deployment.yaml"}, true},
+		{"absolute path, multi source - matching #2", getMultiSourceApp("/source/path", "other/path", "source/path"), []string{"source/path/my-deployment.yaml"}, true},
 		{"absolute path - not matching", getApp("/source/path1", "source/path"), []string{"source/path/my-deployment.yaml"}, false},
+		{"absolute path, multi source - not matching", getMultiSourceApp("/source/path1", "other/path", "source/path"), []string{"source/path/my-deployment.yaml"}, false},
 		{"two relative paths - matching", getApp(".;../shared", "my-app"), []string{"shared/my-deployment.yaml"}, true},
+		{"two relative paths, multi source - matching #1", getMultiSourceApp(".;../shared", "my-app", "other/path"), []string{"shared/my-deployment.yaml"}, true},
+		{"two relative paths, multi source - matching #2", getMultiSourceApp(".;../shared", "my-app", "other/path"), []string{"shared/my-deployment.yaml"}, true},
 		{"two relative paths - not matching", getApp(".;../shared", "my-app"), []string{"README.md"}, false},
+		{"two relative paths, multi source - not matching", getMultiSourceApp(".;../shared", "my-app", "other/path"), []string{"README.md"}, false},
 		{"file relative path - matching", getApp("./my-deployment.yaml", "source/path"), []string{"source/path/my-deployment.yaml"}, true},
+		{"file relative path, multi source - matching #1", getMultiSourceApp("./my-deployment.yaml", "source/path", "other/path"), []string{"source/path/my-deployment.yaml"}, true},
+		{"file relative path, multi source - matching #2", getMultiSourceApp("./my-deployment.yaml", "other/path", "source/path"), []string{"source/path/my-deployment.yaml"}, true},
 		{"file relative path - not matching", getApp("./my-deployment.yaml", "source/path"), []string{"README.md"}, false},
+		{"file relative path, multi source - not matching", getMultiSourceApp("./my-deployment.yaml", "source/path", "other/path"), []string{"README.md"}, false},
 		{"file absolute path - matching", getApp("/source/path/my-deployment.yaml", "source/path"), []string{"source/path/my-deployment.yaml"}, true},
+		{"file absolute path, multi source - matching #1", getMultiSourceApp("/source/path/my-deployment.yaml", "source/path", "other/path"), []string{"source/path/my-deployment.yaml"}, true},
+		{"file absolute path, multi source - matching #2", getMultiSourceApp("/source/path/my-deployment.yaml", "other/path", "source/path"), []string{"source/path/my-deployment.yaml"}, true},
 		{"file absolute path - not matching", getApp("/source/path1/README.md", "source/path"), []string{"source/path/my-deployment.yaml"}, false},
+		{"file absolute path, multi source - not matching", getMultiSourceApp("/source/path1/README.md", "source/path", "other/path"), []string{"source/path/my-deployment.yaml"}, false},
 		{"file two relative paths - matching", getApp("./README.md;../shared/my-deployment.yaml", "my-app"), []string{"shared/my-deployment.yaml"}, true},
+		{"file two relative paths, multi source - matching", getMultiSourceApp("./README.md;../shared/my-deployment.yaml", "my-app", "other-path"), []string{"shared/my-deployment.yaml"}, true},
 		{"file two relative paths - not matching", getApp(".README.md;../shared/my-deployment.yaml", "my-app"), []string{"kustomization.yaml"}, false},
+		{"file two relative paths, multi source - not matching", getMultiSourceApp(".README.md;../shared/my-deployment.yaml", "my-app", "other-path"), []string{"kustomization.yaml"}, false},
 	}
 	for _, tt := range tests {
-		t.Run(tt.name, func(t *testing.T) {
-			if got := appFilesHaveChanged(tt.app, tt.files); got != tt.want {
-				t.Errorf("getAppRefreshPrefix() = %v, want %v", got, tt.want)
+		ttc := tt
+		t.Run(ttc.name, func(t *testing.T) {
+			t.Parallel()
+			if got := appFilesHaveChanged(ttc.app, ttc.files); got != ttc.changeExpected {
+				t.Errorf("getAppRefreshPrefix() = %v, want %v", got, ttc.changeExpected)
 			}
 		})
 	}
 }
 
 func TestAppRevisionHasChanged(t *testing.T) {
-	assert.True(t, appRevisionHasChanged(&v1alpha1.Application{Spec: v1alpha1.ApplicationSpec{
-		Source: v1alpha1.ApplicationSource{},
-	}}, "master", true))
+	getSource := func(targetRevision string) v1alpha1.ApplicationSource {
+		return v1alpha1.ApplicationSource{TargetRevision: targetRevision}
+	}
 
-	assert.False(t, appRevisionHasChanged(&v1alpha1.Application{Spec: v1alpha1.ApplicationSpec{
-		Source: v1alpha1.ApplicationSource{},
-	}}, "master", false))
+	testCases := []struct {
+		name             string
+		source           v1alpha1.ApplicationSource
+		revision         string
+		touchedHead      bool
+		expectHasChanged bool
+	}{
+		{"no target revision, master, touched head", getSource(""), "master", true, true},
+		{"no target revision, master, did not touch head", getSource(""), "master", false, false},
+		{"dev target revision, master, touched head", getSource("dev"), "master", true, false},
+		{"dev target revision, dev, did not touch head", getSource("dev"), "dev", false, true},
+		{"refs/heads/dev target revision, master, touched head", getSource("refs/heads/dev"), "master", true, false},
+		{"refs/heads/dev target revision, dev, did not touch head", getSource("refs/heads/dev"), "dev", false, true},
+		{"env/test target revision, env/test, did not touch head", getSource("env/test"), "env/test", false, true},
+		{"refs/heads/env/test target revision, env/test, did not touch head", getSource("refs/heads/env/test"), "env/test", false, true},
+	}
 
-	assert.False(t, appRevisionHasChanged(&v1alpha1.Application{Spec: v1alpha1.ApplicationSpec{
-		Source: v1alpha1.ApplicationSource{
-			TargetRevision: "dev",
-		},
-	}}, "master", true))
-
-	assert.True(t, appRevisionHasChanged(&v1alpha1.Application{Spec: v1alpha1.ApplicationSpec{
-		Source: v1alpha1.ApplicationSource{
-			TargetRevision: "dev",
-		},
-	}}, "dev", false))
-
-	assert.False(t, appRevisionHasChanged(&v1alpha1.Application{Spec: v1alpha1.ApplicationSpec{
-		Source: v1alpha1.ApplicationSource{
-			TargetRevision: "refs/heads/dev",
-		},
-	}}, "master", true))
-
-	assert.True(t, appRevisionHasChanged(&v1alpha1.Application{Spec: v1alpha1.ApplicationSpec{
-		Source: v1alpha1.ApplicationSource{
-			TargetRevision: "refs/heads/dev",
-		},
-	}}, "dev", false))
-	assert.True(t, appRevisionHasChanged(&v1alpha1.Application{Spec: v1alpha1.ApplicationSpec{
-		Source: v1alpha1.ApplicationSource{
-			TargetRevision: "env/test",
-		},
-	}}, "env/test", false))
-	assert.True(t, appRevisionHasChanged(&v1alpha1.Application{Spec: v1alpha1.ApplicationSpec{
-		Source: v1alpha1.ApplicationSource{
-			TargetRevision: "refs/heads/env/test",
-		},
-	}}, "env/test", false))
+	for _, tc := range testCases {
+		tcc := tc
+		t.Run(tcc.name, func(t *testing.T) {
+			t.Parallel()
+			changed := sourceRevisionHasChanged(tcc.source, tcc.revision, tcc.touchedHead)
+			assert.Equal(t, tcc.expectHasChanged, changed)
+		})
+	}
 }
 
 func Test_affectedRevisionInfo_appRevisionHasChanged(t *testing.T) {
-	appWithRevision := func(targetRevision string) *v1alpha1.Application {
-		return &v1alpha1.Application{Spec: v1alpha1.ApplicationSpec{
-			Source: v1alpha1.ApplicationSource{
-				TargetRevision: targetRevision,
-			},
-		}}
+	sourceWithRevision := func(targetRevision string) v1alpha1.ApplicationSource {
+		return v1alpha1.ApplicationSource{TargetRevision: targetRevision}
 	}
 
 	githubPushPayload := func(branchName string) github.PushPayload {
@@ -417,8 +507,8 @@ func Test_affectedRevisionInfo_appRevisionHasChanged(t *testing.T) {
 		t.Run(testCopy.name, func(t *testing.T) {
 			t.Parallel()
 			_, revisionFromHook, _, _, _ := affectedRevisionInfo(testCopy.hookPayload)
-			if got := appRevisionHasChanged(appWithRevision(testCopy.targetRevision), revisionFromHook, false); got != testCopy.hasChanged {
-				t.Errorf("appRevisionHasChanged() = %v, want %v", got, testCopy.hasChanged)
+			if got := sourceRevisionHasChanged(sourceWithRevision(testCopy.targetRevision), revisionFromHook, false); got != testCopy.hasChanged {
+				t.Errorf("sourceRevisionHasChanged() = %v, want %v", got, testCopy.hasChanged)
 			}
 		})
 	}
@@ -453,7 +543,7 @@ func Test_getWebUrlRegex(t *testing.T) {
 			regexp, err := getWebUrlRegex(testCopy.webURL)
 			assert.NoError(t, err)
 			if matches := regexp.MatchString(testCopy.repo); matches != testCopy.shouldMatch {
-				t.Errorf("appRevisionHasChanged() = %v, want %v", matches, testCopy.shouldMatch)
+				t.Errorf("sourceRevisionHasChanged() = %v, want %v", matches, testCopy.shouldMatch)
 			}
 		})
 	}
